/*
    Munchman for SDL
    Justin Karneges
    infiniti@affinix.com

    This program is licensed under the GNU GPL.  Basically, if you
    make changes or port the game, the new version and source code has
    to be freely available.  That's a reasonable agreement, don't you
    think?  See the file "COPYING" for details.

    Munchman originally was for DOS, and it used quite some DOS code of
    mine.  Now it uses SDL.  There is a bunch of code "commented out" all
    over, mostly references to the old DOS version or removed cheats.

    The game renders all graphics itself (see mmsdl.h) which makes the
    game quite portable, only the final draw to the screen is important.
    This SDL version runs in 16bit hicolor for compatability, as paletted
    resolutions seem to be going out of style.  Munchman contains lots
    of palette animation, so my implementation of SDL in the game
    does an emulation of a palette.  The game *always* renders a 320x200
    game screen, and the zap32() function does the rest (palette
    emulation and resolution scale).

    For speed, the game uses a lot of Intel assembly code in the graphics
    routines.  If you want to compile on a non-Intel system, try defining
    USE_C_ONLY when compiling.  The C versions of the routines are not
    as optimized, but at least it will work.  Let me know your results.

    To compile with gcc, do this:

      gcc mm.c -o mm -lSDL -lpthread

    Note: the game itself does not use threading, but SDL needs it (at
          least on my system)

    Have fun.
*/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<math.h>
#include<time.h>
#include"mmsdl.h"
#include"crupal.h"

#define BLOCK   (32*32)

#define NORM    0

#define UP      1
#define DOWN    2
#define LEFT    3
#define RIGHT   4

#define YES     1
#define NO      0

#define GHOSTSPEED 2

struct DUDE {
        int x;
        int y;
        int rx, ry;
        int dir;
        int action;
        char dead;
        int dead_time;
        int moving;
        int n;
        int pic;
        char up,down,left,right;
        int movspeed;
        char falling;
        int fallcount;
};

char animon[9];
int anim[9];
int lev4count;
int fade_in;

int game_loops;
char skip_draw;
char stablizer = 1;

char pal2[24];
char pal3[768];

int scrollspeed = 8;
int lives;
char not_move;
int sx, sy;
int xend, yend;

struct {
        int x;
        int y;
        int ghosts;
        int startx;
        int starty;
} stat[6];

struct DUDE ch[5];

main(int argc, char *argv[])
{
        FILE *f;
        char *dbuf;
        int n, n2;
        char ok;
        char animup;
        char animcount;
        char mem[BLOCK * 24];
        char wall[BLOCK * 25];
        char *backg;

        char *pactitle;
        char *paclose;
        char *pacend;

        char paclife[64];
        char pac_erase[64];
        time_t t;
        int start_time, now_time;

        char eat_mode;
        int eat_time;
        int lev;
        char backpal[768];

        int fullscreen = 0, doublesize = 1;

        char map[6][88][42] = {
                {
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1 },
                { 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1 },
                { 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1 },
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
                },

                {
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 1 },
                { 1, 0, 1, 2, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1 },
                { 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1 },
                { 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1 },
                { 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1 },
                { 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1 },
                { 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
                },

                {
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 2, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 2, 1 },
                { 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 2, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
                },

                {
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1 },
                { 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1 },
                { 1, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1 },
                { 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 2, 0, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 2, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1 },
                { 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
                },

                {
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 2, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 2, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 1, 1, 0, 1, 1, 1 },
                { 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 2, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1 },
                { 1, 0, 0, 0, 1, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 2, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1 },
                { 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1 },
                { 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1 },
                { 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1 },
                { 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1 },
                { 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1 },
                { 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1 },
                { 1, 0, 0, 0, 0, 0, 1, 2, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 1, 1, 0, 1, 2, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 2, 1 },
                { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
                },
                


                {
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 0, 0, 0, 0, 0, 0, 0, 1, 3, 3, 1, 0, 0, 0, 0, 0, 1, 1, 1, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 3, 3, 3, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
            /*15*/    

                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 3, 1, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3, 1, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 0, 0, 0, 0, 1, 3, 1, 1, 1, 0, 1, 1, 1, 3, 3, 3, 3, 3, 1, 0, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 1, 0, 1, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 0, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 3, 3, 1, 1, 1, 1, 1, 1, 0, 1, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 1, 0, 0, 0, 0, 1, 3, 3, 1, 0, 0, 0, 0, 0, 0, 1, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 1, 1, 1, 0, 1, 3, 3, 1, 1, 1, 1, 0, 1, 3, 3, 1, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 1, 0, 0, 0, 1, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 1, 0, 0, 0, 0, 0, 5, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 3, 3, 3, 3, 5, 0, 0, 1, 3, 3, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
            /* 17 */

                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 1, 1, 1, 0, 1, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 1, 0, 1, 1, 3, 3, 3, 3, 3, 1, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 1, 0, 0, 1, 1, 1, 3, 3, 3, 1, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3, 3, 1, 1, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 0, 3, 3, 3, 0, 1, 1, 0, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 0, 1, 0, 1, 3, 1, 0, 1, 3, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 0, 3, 3, 3, 0, 1, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 0, 1, 0, 1, 3, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 0, 5, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
            /*23*/
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 1, 1, 1, 1, 3, 3, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 1, 1, 3, 3, 3, 3, 3, 3, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 1, 0, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 1, 3, 3, 0, 0, 0, 0, 0, 3, 1, 1, 1, 1, 3, 3, 3, 3, 1, 1, 0, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 1, 3, 0, 0, 3, 3, 0, 0, 0, 0, 0, 0, 1, 1, 3, 3, 3, 1, 3, 0, 3, 3, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 1, 3, 0, 0, 3, 3, 0, 3, 0, 3, 1, 1, 1, 1, 3, 3, 3, 1, 3, 0, 0, 0, 0, 0, 3, 3, 3, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 1, 1, 3, 0, 3, 3, 0, 3, 0, 3, 3, 3, 3, 1, 3, 3, 3, 1, 3, 0, 3, 3, 3, 0, 0, 0, 3, 3, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 3, 0, 3, 3, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 3, 0, 0, 0, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 0, 0, 0, 0, 3, 0, 3, 3, 0, 3, 1, 3, 0, 3, 3, 1, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 3, 3, 0, 0, 0, 0, 0, 0, 0, 3, 1, 3, 0, 0, 3, 1, 3, 3, 0, 3, 0, 3, 0, 3, 0, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 0, 0, 3, 3, 3, 3, 3, 3, 0, 3, 3, 1, 3, 3, 0, 3, 1, 3, 3, 0, 3, 0, 3, 0, 0, 0, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 1, 3, 3, 0, 3, 1, 1, 3, 3, 0, 0, 0, 0, 0, 3, 3, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 1, 3, 3, 4, 3, 3, 1, 1, 3, 3, 3, 3, 3, 3, 3, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 1, 1, 1, 1, 1, 1, 1, 1, 1, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 },
                { 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3 }
            /*18 */

                }
        };
        char pellet[88][42];
        int numpellets;
        
        
#ifdef __BEOS__
int counti;
char datapath[256];
for (counti=strlen(argv[0]) ; argv[0][counti]!='/' ; counti--);
strncpy(datapath, argv[0], counti);
chdir(datapath);
#endif

        // find commandline args
        for(n = 1; n < argc; ++n) {
                if(!strcmp(argv[n], "--fullscreen"))
                        fullscreen = 1;
                if(!strcmp(argv[n], "--small"))
                        doublesize = 0;
                if(!strcmp(argv[n], "--notimer"))
                        stablizer = 0;
        }

        //struct DUDE ch[5];

        for(n = 0; n < 64; ++n)
                pac_erase[n] = 0;

        dbuf = malloc(64000);
        backg = malloc(64000);
        pactitle = malloc(64000);
        paclose = malloc(64000);
        pacend = malloc(64000);
        if(!dbuf || !backg || !pactitle || !paclose || !pacend) {
                printf("mm: not enough memory\n");
                exit(1);
        }

        f = fopen("mm.dat", "rb");
        if(!f) {
                fprintf(stderr, "mm: error opening mm.dat\n");
                exit(1);
        }
        fread(mem, BLOCK * 24, 1, f);
        fread(wall, BLOCK * 24, 1, f);
        fread(backg, 64000, 1, f);
        fread(wall + BLOCK * 24, BLOCK, 1, f);
        fread(paclife, 64, 1, f);
        fread(pactitle, 64000, 1, f);
        fread(paclose, 64000, 1, f);
        fread(pacend, 64000, 1, f);
        fclose(f);

/*        f = fopen("gr.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(mem, BLOCK * 24, 1, f);
        fclose(f);
        f = fopen("wall.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(wall, BLOCK * 24, 1, f);
        fclose(f);
        f = fopen("backg.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(backg, 64000, 1, f);
        fclose(f);
        f = fopen("spring.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(wall + BLOCK * 24, BLOCK, 1, f);
        fclose(f);
        f = fopen("paclife.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(paclife, 64, 1, f);
        fclose(f);
        f = fopen("title.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(pactitle, 64000, 1, f);
        fclose(f);
        f = fopen("lose.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(paclose, 64000, 1, f);
        fclose(f);
        f = fopen("end.gra", "rb");
        if(!f) {
                printf("Missing a file!\n");
                exit(1);
        }
        fread(pacend, 64000, 1, f);
        fclose(f);
*/

        stat[0].x = 21;
        stat[0].y = 11;
        stat[0].startx = 320;
        stat[0].starty = 96;
        stat[0].ghosts = 4;

        stat[1].x = 21;
        stat[1].y = 11;
        stat[1].startx = 320;
        stat[1].starty = 96;
        stat[1].ghosts = 4;

        stat[2].x = 42;
        stat[2].y = 22;
        stat[2].startx = 320;
        stat[2].starty = 96;
        stat[2].ghosts = 4;

        stat[3].x = 21;
        stat[3].y = 11;
        stat[3].startx = 320;
        stat[3].starty = 96;
        stat[3].ghosts = 2;

        stat[4].x = 42;
        stat[4].y = 22;
        stat[4].startx = 32 * (21-1);
        stat[4].starty = 32 * (11-1);
        stat[4].ghosts = 4;

        stat[5].x = 42;
        stat[5].y = 72;
        stat[5].startx = 32 * 5;
        stat[5].starty = 32 * 6;
        stat[5].ghosts = 4;

        sdl_init(doublesize, fullscreen);
//        install_keybint();
//        install_timer(TIMER_60);
//        vga256();

        //setpal(gamepal);

        for(n = 0; n < 768; ++n) {
                pal3[n] = 0;
        }
        setpal(pal3);
        put(0, 0, 320, 200, pactitle, dbuf);
        zap32(dbuf);
        for(n2 = 0; n2 < 64; ++n2) {
                zap32(dbuf);
                wait_r();
                for(n = 0; n < 256; ++n) {
                        if(pal3[n*3] < gamepal[n*3])
                                ++pal3[n*3];
                        if(pal3[n*3+1] < gamepal[n*3+1])
                                ++pal3[n*3+1];
                        if(pal3[n*3+2] < gamepal[n*3+2])
                                ++pal3[n*3+2];
                }

                setpal(pal3);
        }
        zap32(dbuf);

        updatekeys();
        while(!keypress[28] && !keypress[1])
                updatekeys();
        if(keypress[1])
                goto end;
        for(n2 = 0; n2 < 64; ++n2) {
                zap32(dbuf);
                wait_r();
                for(n = 0; n < 256; ++n) {
                        if(pal3[n*3] > 0)
                                --pal3[n*3];
                        if(pal3[n*3+1] > 0)
                                --pal3[n*3+1];
                        if(pal3[n*3+2] > 0)
                                --pal3[n*3+2];
                }

                setpal(pal3);
        }
        zap32(dbuf);

        lives = 10;

        /*if(argc > 1) {
                lev = atoi(argv[1]) - 1;
                if(lev > 6 || lev < 0)
                        lev = 0;
        }
        else*/
                lev = 0;

        srand(time(NULL));
//        gettime(&t);
//        srand(t.ti_sec);

        for(n = 0; n < BLOCK * 16; ++n) {
                if(mem[n + BLOCK * 8] >= 96)
                        mem[n + BLOCK * 8] += 64;
        }
        memcpy(pal2, gamepal + (32+8) * 3, 8 * 3);
        memcpy(gamepal + 160 * 3, gamepal + 96 * 3, 16 * 3);
        //setpal(gamepal);

        memcpy(backpal, gamepal, 768);

        again:
        //stop_ticks();

        if(lev >= 6) {
                memcpy(gamepal, backpal, 768);
                setpal(gamepal);
                for(n = 0; n < 256; ++n)
                        pal3[n] = 0;
                setpal(pal3);
                put(0, 0, 320, 200, pacend, dbuf);
                place(144, 120, 32, 32, wall + BLOCK * 3, dbuf);
                zap32(dbuf);
                for(n2 = 0; n2 < 64; ++n2) {
                        wait_r();
                        for(n = 0; n < 256; ++n) {
                                if(pal3[n*3] < gamepal[n*3])
                                        ++pal3[n*3];
                                if(pal3[n*3+1] < gamepal[n*3+1])
                                        ++pal3[n*3+1];
                                if(pal3[n*3+2] < gamepal[n*3+2])
                                        ++pal3[n*3+2];
                        }

                        setpal(pal3);
                        zap32(dbuf);
                }
                for(n = 640; n > 0; n -= 2) {
                        n2 = (n / 16)&1 ? 8: 0;
                        put(0, 0, 320, 200, pacend, dbuf);
                        if(n > 208)
                                place(144, 120, 32, 32, wall + BLOCK * 3, dbuf);
                        place(n - 64, 120, 32, 32, mem + BLOCK * (8+n2), dbuf);
                        zap32(dbuf);
                }

                updatekeys();
                while(!keypress[28] && !keypress[1])
                        updatekeys();
                goto end;
        }

        for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                while(1) {
                        ch[n].rx = rand() % stat[lev].x;
                        ch[n].ry = rand() % stat[lev].y;
                        if(map[lev][ch[n].ry][ch[n].rx] == 0) {
                                break;
                        }
                }
                ch[n].x = ch[n].rx * 32;
                ch[n].y = ch[n].ry * 32;
                ch[n].pic = n - 1;
                ch[n].dead = 0;
                ch[n].action = NORM;
                ch[n].dir = RIGHT;
                ch[n].moving = 0;
                ch[n].movspeed = GHOSTSPEED;
                ch[n].falling = 0;
        }

        setpale(159, 0, 0, 0);
        setpale(158, 0, 0, 0);

        animup = 1;
        animcount = 0;

        ch[0].x = stat[lev].startx;
        ch[0].y = stat[lev].starty;
        ch[0].action = NORM;
        ch[0].dir = RIGHT;
        ch[0].moving = 0;
        ch[0].movspeed = 2;
        ch[0].falling = 0;
        ch[0].dead = 0;

        numpellets = 0;
        for(n2 = 0; n2 < stat[lev].y; ++n2) {
                for(n = 0; n < stat[lev].x; ++n) {
                        if(map[lev][n2][n] == 2) {
                                pellet[n2][n] = 2;
                                ++numpellets;
                        }
                        else if(map[lev][n2][n] == 0) {
                                pellet[n2][n] = 1;
                                ++numpellets;
                        }
                        else
                                pellet[n2][n] = 0;
                }
        }

        sx = 0;
        sy = 0;
        memset(dbuf, 0, 64000);
        game_loops = 0;
        skip_draw = NO;
        eat_mode = NO;
        memcpy(gamepal, backpal, 768);
        setpal(gamepal);

        for(n = 0; n < 9; ++n) {
                animon[n] = 0;
                anim[n] = 0;
        }
        lev4count = 0;
        not_move = 1;

        if(lev > 0) {
                fade_in = 64;
                for(n = 0; n < 768; ++n)
                        pal3[n] = 63;
        }

        ticks = 0;
//        start_ticks();
        start_time = get_ticks();

        while(1) {
                if(fade_in > 0) {
                        --fade_in;
                        for(n = 0; n < 256; ++n) {
                                if(pal3[n*3] > gamepal[n*3])
                                        --pal3[n*3];
                                if(pal3[n*3+1] > gamepal[n*3+1])
                                        --pal3[n*3+1];
                                if(pal3[n*3+2] > gamepal[n*3+2])
                                        --pal3[n*3+2];
                        }
                        setpal(pal3);
                }

                if(lives <= 0) {
                        for(n = 0; n < 256; ++n)
                                pal3[n] = 0;
                        setpal(pal3);
                        put(0, 0, 320, 200, paclose, dbuf);
                        zap32(dbuf);
                        for(n2 = 0; n2 < 64; ++n2) {
                                wait_r();
                                for(n = 0; n < 256; ++n) {
                                        if(pal3[n*3] < gamepal[n*3])
                                                ++pal3[n*3];
                                        if(pal3[n*3+1] < gamepal[n*3+1])
                                                ++pal3[n*3+1];
                                        if(pal3[n*3+2] < gamepal[n*3+2])
                                                ++pal3[n*3+2];
                                }

                                setpal(pal3);
                                zap32(dbuf);
                        }
                        updatekeys();
                        while(!keypress[28] && !keypress[1])
                                updatekeys();
                        goto end;
                }
                for(n = 0; n < stat[lev].ghosts + 1; ++n) {
                        ch[n].rx = ch[n].x/32;
                        ch[n].ry = ch[n].y/32;
                }

                /* ghost on square */
                for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                        if(!(ch[n].x%32) && !(ch[n].y%32)) {
                                if(lev == 5 && !ch[n].falling) {
                                        if(map[lev][ch[n].ry][ch[n].rx] == 4) {
                                                ch[n].x = stat[lev].startx;
                                                ch[n].y = stat[lev].starty;
                                                ch[n].dir = RIGHT;
                                                ch[n].action = NORM;
                                                ch[n].moving = 0;
                                        }
                                        else if(ch[n].rx == 12 && ch[n].ry == 9) {
                                                ch[n].moving = 0;
                                                ch[n].action = NORM;
                                                ch[n].fallcount = 32; //40; //64;
                                                ch[n].falling = 1;
                                        }
                                        else if(ch[n].rx == 14 && ch[n].ry == 30) {
                                                ch[n].moving = 0;
                                                ch[n].action = NORM;
                                                ch[n].fallcount = 32;
                                                ch[n].falling = 2;
                                        }
                                        else if(ch[n].rx == 26 && ch[n].ry == 29) {
                                                ch[n].moving = 0;
                                                ch[n].action = NORM;
                                                ch[n].fallcount = 32;
                                                ch[n].falling = 1;
                                        }
                                        else if(ch[n].rx == 16 && ch[n].ry == 51) {
                                                ch[n].moving = 0;
                                                ch[n].action = NORM;
                                                ch[n].fallcount = 32;
                                                ch[n].falling = 1;
                                        }
                                        else if(ch[n].rx == 20 && ch[n].ry == 53) {
                                                ch[n].moving = 0;
                                                ch[n].action = NORM;
                                                ch[n].fallcount = 32;
                                                ch[n].falling = 2;
                                        }
                                }
                        }
                }

                /* if you're centered on a square: */
                if(!(ch[0].x%32) && !(ch[0].y%32) && !ch[0].dead) {
                        if(lev == 5 && !ch[0].falling) {
                                if(map[lev][ch[0].ry][ch[0].rx] == 4) {
                                        ch[0].x = stat[lev].startx;
                                        ch[0].y = stat[lev].starty;
                                        ch[0].dir = RIGHT;
                                        ch[0].action = NORM;
                                        ch[0].moving = 0;
                                }
                                else if(ch[0].rx == 12 && ch[0].ry == 9) {
                                        ch[0].moving = 0;
                                        ch[0].action = NORM;
                                        ch[0].fallcount = 32; //40; //64;
                                        ch[0].falling = 1;
                                }
                                else if(ch[0].rx == 14 && ch[0].ry == 30) {
                                        ch[0].moving = 0;
                                        ch[0].action = NORM;
                                        ch[0].fallcount = 32;
                                        ch[0].falling = 2;
                                }
                                else if(ch[0].rx == 26 && ch[0].ry == 29) {
                                        ch[0].moving = 0;
                                        ch[0].action = NORM;
                                        ch[0].fallcount = 32;
                                        ch[0].falling = 1;
                                }
                                else if(ch[0].rx == 16 && ch[0].ry == 51) {
                                        ch[0].moving = 0;
                                        ch[0].action = NORM;
                                        ch[0].fallcount = 32;
                                        ch[0].falling = 1;
                                }
                                else if(ch[0].rx == 20 && ch[0].ry == 53) {
                                        ch[0].moving = 0;
                                        ch[0].action = NORM;
                                        ch[0].fallcount = 32;
                                        ch[0].falling = 2;
                                }

                                else if(map[lev][ch[0].ry][ch[0].rx] == 3 && !keypress[5]) {
                                        ch[0].action = NORM;
                                        ch[0].moving = 0;
                                        ch[0].dead = 2;

                                        if(eat_mode) {
                                                eat_mode = NO;
                                                memcpy(gamepal + 160 * 3, gamepal + 96 * 3, 16 * 3);
                                                setpal(gamepal);
                                        }
                                }
                        }
                        if(lev == 3 && map[lev][ch[0].ry][ch[0].rx] == 1 && !keypress[5]) {
                                        ch[0].action = NORM;
                                        ch[0].moving = 0;
                                        ch[0].dead = 2;

                                        if(eat_mode) {
                                                eat_mode = NO;
                                                memcpy(gamepal + 160 * 3, gamepal + 96 * 3, 16 * 3);
                                                setpal(gamepal);
                                        }
                        }
                        if(pellet[ch[0].ry][ch[0].rx] == 2) {
                                eat_mode = YES;
                                eat_time = 600;
                                memcpy(gamepal + 160 * 3, gamepal + 32 * 3, 16 * 3);
                        }
                        if(pellet[ch[0].ry][ch[0].rx] > 0) {
                                pellet[ch[0].ry][ch[0].rx] = 0;
                                --numpellets;
                        }
                }

                if(numpellets == 0) {
                        for(n = 0; n < 64; ++n) {
                                for(n2 = 0; n2 < 256; ++n2) {
                                        if(gamepal[n2 * 3] < 63)
                                                ++gamepal[n2 * 3];
                                        if(gamepal[n2 * 3 + 1] < 63)
                                                ++gamepal[n2 * 3 + 1];
                                        if(gamepal[n2 * 3 + 2] < 63)
                                                ++gamepal[n2 * 3 + 2];
                                }
                                wait_r();
                                setpal(gamepal);
                                zap32(dbuf);
                        }
                        /*for(n = 0; n < 64; ++n) {
                                for(n2 = 0; n2 < 256; ++n2) {
                                        if(gamepal[n2 * 3] > 0)
                                                --gamepal[n2 * 3];
                                        if(gamepal[n2 * 3 + 1] > 0)
                                                --gamepal[n2 * 3 + 1];
                                        if(gamepal[n2 * 3 + 2] > 0)
                                                --gamepal[n2 * 3 + 2];
                                }
                                wait_r();
                                setpal(gamepal);
                        }*/
                        ++lev;
                        goto again;
                }

                if(eat_mode) {
                        if(eat_time > 0)
                                --eat_time;
                        else {
                                eat_mode = NO;
                                memcpy(gamepal + 160 * 3, gamepal + 96 * 3, 16 * 3);
                                setpal(gamepal);
                        }
                }

                if(eat_mode && !(ch[0].n % 4)) {
                        ch[0].movspeed = 4;
                }
                else {
                        ch[0].movspeed = 2;
                }

                for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                        if(!ch[n].dead && !(ch[n].n % 4)) {
                                ch[n].movspeed = GHOSTSPEED;
                        }
                        else {
                                ch[n].movspeed = 2;
                        }
                }

                if(!skip_draw) {
                        clip.y2 = 191;
                        if(sx%32 == 0)
                                xend = 10;
                        else
                                xend = 11;
                        if(sy%32 == 0)
                                yend = 6;
                        else
                                yend = 7;
                        if(lev == 3 || lev == 5)
                                put(0, 0, 320, 200, backg, dbuf);

                        if(ch[0].dead == 2) {
                                if(ch[0].dir == RIGHT)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (9 * BLOCK), dbuf);
                                if(ch[0].dir == LEFT)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (8 * BLOCK), dbuf);
                                if(ch[0].dir == UP)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (10 * BLOCK), dbuf);
                                if(ch[0].dir == DOWN)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (11 * BLOCK), dbuf);
                        }

                        for(n2 = 0; n2 < yend; ++n2) {
                                for(n = 0; n < xend; ++n) {
                                        if(lev == 3) {
                                                place(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, map[lev][n2+sy/32][n+sx/32] == 1 ? wall + (BLOCK) + (lev*4*BLOCK): wall + (lev*4*BLOCK), dbuf);
                                        }
                                        else if(lev == 5) {
                                                if(map[lev][n2+sy/32][n+sx/32] == 4) {
                                                        place(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, wall + (lev*4*BLOCK), dbuf);
                                                        place(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, wall + BLOCK*24, dbuf);
                                                }
                                                else if(map[lev][n2+sy/32][n+sx/32] != 3 && map[lev][n2+sy/32][n+sx/32] != 5)
                                                        place(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, map[lev][n2+sy/32][n+sx/32] == 1 ? wall + (BLOCK) + (lev*4*BLOCK): wall + (lev*4*BLOCK), dbuf);
                                        }
                                        else {
                                                put(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, map[lev][n2+sy/32][n+sx/32] == 1 ? wall + (BLOCK) + (lev*4*BLOCK): wall + (lev*4*BLOCK), dbuf);
                                        }
                                        if(pellet[n2+sy/32][n+sx/32] == 1)
                                                place(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, wall + BLOCK * 2 + (lev*4*BLOCK), dbuf);
                                        else if(pellet[n2+sy/32][n+sx/32] == 2)
                                                place(n * 32 - sx%32, n2 * 32 - sy%32, 32, 32, wall + BLOCK * 3 + (lev*4*BLOCK), dbuf);
                                }
                        }
                        if(ch[0].dir == RIGHT) {
                                if(ch[0].n < 16)
                                        ch[0].pic = 9;
                                else
                                        ch[0].pic = 17;
                        }
                        if(ch[0].dir == LEFT) {
                                if(ch[0].n < 16)
                                        ch[0].pic = 8;
                                else
                                        ch[0].pic = 16;
                        }
                        if(ch[0].dir == UP) {
                                if(ch[0].n < 16)
                                        ch[0].pic = 10;
                                else
                                        ch[0].pic = 18;
                        }
                        if(ch[0].dir == DOWN) {
                                if(ch[0].n < 16)
                                        ch[0].pic = 11;
                                else
                                        ch[0].pic = 19;
                        }

                        for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                                if(!ch[n].dead)
                                        place(ch[n].x - sx, ch[n].y - sy, 32, 32, mem + ((n-1) * BLOCK), dbuf);
                                if(ch[n].dir == DOWN)
                                        ch[n].pic = 4;
                                else if(ch[n].dir == UP)
                                        ch[n].pic = 5;
                                else if(ch[n].dir == LEFT)
                                        ch[n].pic = 6;
                                else if(ch[n].dir == RIGHT)
                                        ch[n].pic = 7;
                                place(ch[n].x - sx, ch[n].y - sy, 32, 32, mem + ch[n].pic * BLOCK, dbuf);
                        }

                        if(ch[0].dead == 0) {
                                if(not_move)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (12 * BLOCK), dbuf);
                                else
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (ch[0].pic * BLOCK), dbuf);
                        }
                        else if(ch[0].dead == 1) {
                                if(ch[0].n < 8)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (8 * BLOCK), dbuf);
                                else if(ch[0].n < 16)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (10 * BLOCK), dbuf);
                                else if(ch[0].n < 24)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (9 * BLOCK), dbuf);
                                else if(ch[0].n < 32)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (11 * BLOCK), dbuf);

                                else if(ch[0].n < 40)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (8 * BLOCK), dbuf);
                                else if(ch[0].n < 48)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (10 * BLOCK), dbuf);
                                else if(ch[0].n < 56)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (9 * BLOCK), dbuf);
                                else if(ch[0].n < 64)
                                        place(ch[0].x - sx, ch[0].y - sy, 32, 32, mem + (20 * BLOCK), dbuf);
                        }

                        clip.y2 = 199;
                        for(n = 0; n < lives; ++n) {
                                put(n * 8, 192, 8, 8, paclife, dbuf);
                        }
                        for(n = lives; n < 40; ++n) {
                                put(n * 8, 192, 8, 8, pac_erase, dbuf);
                        }

                        wait_r();
                        zap32(dbuf);
                }

                if(!fade_in) {
                        setpale(156, animcount, 0, 0);
                        setpale(158, 0, animcount, 0);
                        setpale(159, 0, animcount, animcount);
                
                        if(lev == 2) {
                                setpale(199, animcount, animcount, 0);
                                for(n = 7; n >= 1; --n) {
                                        pal2[n * 3] = pal2[(n - 1) * 3];
                                        pal2[n * 3 + 1] = pal2[(n - 1) * 3 + 1];
                                        pal2[n * 3 + 2] = pal2[(n - 1) * 3 + 2];
                                }
                                pal2[0] = pal2[7 * 3];
                                pal2[1] = pal2[7 * 3 + 1];
                                pal2[2] = pal2[7 * 3 + 2];
                                for(n = 0; n < 16; ++n) {
                                        setpale(200 + n, pal2[n * 3], pal2[n * 3 + 1], pal2[n * 3 + 2]);
                                }
                        }
                        else if(lev == 4) {
                                if(lev4count >= 0)
                                        ++lev4count;
                                if(lev4count == 250) {
                                        animon[0] = 1;
                                        lev4count = -1;
                                }

                                if(animon[0]) {
                                        setpale(208, anim[0], anim[0], anim[0]);
                                        if(anim[0] == 56) {
                                                animon[1] = 1;
                                        }
                                }
                                if(animon[1]) {
                                        setpale(209, 0, 0, anim[1]);
                                        if(anim[1] == 56) {
                                                animon[3] = 1;
                                        }
                                }
                                if(animon[2]) {
                                        setpale(210, 0, anim[2], 0);
                                        if(anim[2] == 56)
                                                animon[4] = 1;
                                }
                                if(animon[3]) {
                                        setpale(211, 0, anim[3], anim[3]);
                                        if(anim[3] == 56)
                                                animon[2] = 1;
                                }
                                if(animon[4]) {
                                        setpale(212, anim[4], 0, 0);
                                        if(anim[4] == 56)
                                                animon[6] = 1;
                                }

                                if(animon[5]) {
                                        setpale(213, anim[5], 0, anim[5]);
                                        if(anim[5] == 56)
                                                animon[7] = 1;
                                }
                                if(animon[6]) {
                                        setpale(214, anim[6], anim[6], 0);
                                        if(anim[6] == 56)
                                                animon[5] = 1;
                                }
                                if(animon[7]) {
                                        setpale(215, anim[7], anim[7], anim[7]);
                                        if(anim[7] == 56)
                                                animon[8] = 1;
                                }
                                if(animon[8]) {
                                        setpale(216, anim[8], anim[8], anim[8]);
                                        if(anim[8] == 56)
                                                lev4count = 0;
                                }

                                setpale(217, animcount, animcount, 0);

                                for(n = 0; n < 9; ++n) {
                                        if(animon[n] == 1)
                                                anim[n] += 8;
                                        else if(animon[n] == 2)
                                                anim[n] -= 8;

                                        if(anim[n] >= 63) {
                                                animon[n] = 2;
                                        }
                                        if(anim[n] == 0 && animon[n] == 2) {
                                                animon[n] = 0;
                                                setpale(208 + n, 0, 0, 0);
                                        }
                                }
                        }

                        if(eat_mode) {
                                if(lev == 3) {
                                        setpale(1, 0, animcount, 0);
                                }
                                else
                                        setpale(0, 0, animcount, 0);
                                for(n = 15; n >= 1; --n) {
                                        gamepal[(160 + n) * 3] = gamepal[(160 + n - 1) * 3];
                                        gamepal[(160 + n) * 3 + 1] = gamepal[(160 + n - 1) * 3 + 1];
                                        gamepal[(160 + n) * 3 + 2] = gamepal[(160 + n - 1) * 3 + 2];
                                }
                                gamepal[(160 * 3)] = gamepal[(160 + 15) * 3];
                                gamepal[(160 * 3) + 1] = gamepal[(160 + 15) * 3 + 1];
                                gamepal[(160 * 3) + 2] = gamepal[(160 + 15) * 3 + 2];
                                for(n = 0; n < 16; ++n) {
                                        setpale(160 + n, gamepal[(160 + n) * 3], gamepal[(160 + n) * 3 + 1], gamepal[(160 + n) * 3 + 2]);
                                }
                        }
                }
                skip_draw = NO;


                if(animup) {
                        if(animcount == 63)
                                animup = 0;
                        else
                                ++animcount;
                }
                else {
                        if(animcount == 0)
                                animup = 1;
                        else
                                --animcount;
                }

                for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                        if(ch[n].dead) {
                                if(ch[n].dead_time > 0)
                                        --ch[n].dead_time;
                                else {
                                        ch[n].dead = 0;
                                        ch[n].x = stat[lev].startx;
                                        ch[n].y = stat[lev].starty;
                                        ch[n].dir = DOWN;
                                        ch[n].action = NORM;
                                        ch[n].moving = 0;
                                        ch[n].falling = 0;
                                }
                        }
                }

                for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                        if(ch[n].dead)
                                continue;
                        if( ( (ch[0].x + 5 <= ch[n].x + 27 && ch[0].x + 5 >= ch[n].x + 5) ||
                              (ch[0].x + 27 >= ch[n].x + 5 && ch[0].x + 27 <= ch[n].x + 27) ) &&
                            ( (ch[0].y + 5 <= ch[n].y + 32 && ch[0].y + 5 >= ch[n].y + 5) ||
                              (ch[0].y + 27 >= ch[n].y + 5 && ch[0].y + 27 <= ch[n].y + 27) ) && !not_move) {

                                if(eat_mode) {
                                        ch[n].dead = 1;
                                        ch[n].dead_time = 900;
                                }
                                else if(!keypress[5] && ch[0].dead == 0) {
                                        ch[0].dead = 1;
                                        ch[0].n = 0;
                                }
                        }
                }
                if(ch[0].dead == 1) {
                        if(ch[0].n < 64) {
                                ++ch[0].n;
                        }
                        else {
                                ch[0].x = stat[lev].startx;
                                ch[0].y = stat[lev].starty;
                                ch[0].dir = RIGHT;
                                ch[0].action = NORM;
                                ch[0].moving = 0;
                                ch[0].falling = 0;
                                ch[0].dead = 0;
                                if(lives > 0)
                                        --lives;
                                not_move = 1;
                        }
                }
                else if(ch[0].dead == 2) {
                        if(ch[0].n < 64) {
                                if(ch[0].n < 16)
                                        ch[0].y += 2;
                                else if(ch[0].n < 32)
                                        ch[0].y += 4;
                                else if(ch[0].n < 48)
                                        ch[0].y += 6;
                                else if(ch[0].n < 64)
                                        ch[0].y += 8;

                                if(ch[0].dir == RIGHT)
                                        ch[0].x += 2;
                                else if(ch[0].dir == LEFT)
                                        ch[0].x -= 2;
                                ++ch[0].n;
                        }
                        else {
                                ch[0].x = stat[lev].startx;
                                ch[0].y = stat[lev].starty;
                                ch[0].dir = RIGHT;
                                ch[0].action = NORM;
                                ch[0].moving = 0;
                                ch[0].falling = 0;
                                ch[0].dead = 0;
                                if(lives > 0)
                                        --lives;
                                not_move = 1;
                        }
                }

                for(n = 0; n < stat[lev].ghosts + 1; ++n) {
                        ch[n].rx = ch[n].x/32;
                        ch[n].ry = ch[n].y/32;
                }

                for(n = 0; n < stat[lev].ghosts + 1; ++n) {
                        if(n == 0 && ch[0].dead) {
                                ch[0].up = 1;
                                ch[0].down = 1;
                                ch[0].left = 1;
                                ch[0].right = 1;
                                continue;
                        }
                        if(lev == 3 && n == 0) {
                                ch[0].up = 1;
                                ch[0].down = 1;
                                ch[0].left = 1;
                                ch[0].right = 1;
                                continue;
                        }

                        if(map[lev][ch[n].ry - 1][ch[n].rx] == 1)
                                ch[n].up = 0;
                        else
                                ch[n].up = 1;
                        if(lev == 5 && n > 0) {
                                if(map[lev][ch[n].ry - 1][ch[n].rx] == 3)
                                        ch[n].up = 0;
                        }

                        if(map[lev][ch[n].ry + 1][ch[n].rx] == 1)
                                ch[n].down = 0;
                        else
                                ch[n].down = 1;
                        if(lev == 5 && n > 0) {
                                if(map[lev][ch[n].ry + 1][ch[n].rx] == 3)
                                        ch[n].down = 0;
                        }

                        if(map[lev][ch[n].ry][ch[n].rx - 1] == 1)
                                ch[n].left = 0;
                        else
                                ch[n].left = 1;
                        if(lev == 5 && n > 0) {
                                if(map[lev][ch[n].ry][ch[n].rx - 1] == 3)
                                        ch[n].left = 0;
                        }

                        if(map[lev][ch[n].ry][ch[n].rx + 1] == 1)
                                ch[n].right = 0;
                        else
                                ch[n].right = 1;
                        if(lev == 5 && n > 0) {
                                if(map[lev][ch[n].ry][ch[n].rx + 1] == 3)
                                        ch[n].right = 0;
                        }
                }

                updatekeys();
                if(keypress[1])
                        break;
                if(keypress[2]) {
                        stablizer = 0;
                        //stop_ticks();
                }
                if(keypress[3]) {
                        stablizer = 1;
                        //start_ticks();
                }
                if(keypress[4] && !(ch[0].n % 4))
                        ch[0].movspeed = 4;

                if(keypress[6]) {
                        if(keypress[72]) {
                                sy -= 10;
                                if(sy < 0)
                                        sy = 0;
                        }
                        if(keypress[75]) {
                                sx -= 10;
                                if(sx < 0)
                                        sx = 0;
                        }
                        if(keypress[77])
                                sx += 10;
                        if(keypress[80])
                                sy += 10;
                }
                if(!ch[0].moving && !ch[0].falling && !keypress[6] && !ch[0].dead) {
                        if(ch[0].action != NORM) {
                                if(ch[0].dir == UP && ch[0].up)
                                        ch[0].action = UP;
                                else if(ch[0].dir == DOWN && ch[0].down)
                                        ch[0].action = DOWN;
                                else if(ch[0].dir == LEFT && ch[0].left)
                                        ch[0].action = LEFT;
                                else if(ch[0].dir == RIGHT && ch[0].right)
                                        ch[0].action = RIGHT;
                                else
                                        ch[0].action = NORM;
                        }

                        if(keypress[72] && ch[0].up)
                                ch[0].action = UP;
                        if(keypress[80] && ch[0].down)
                                ch[0].action = DOWN;
                        if(keypress[75] && ch[0].left)
                                ch[0].action = LEFT;
                        if(keypress[77] && ch[0].right)
                                ch[0].action = RIGHT;
                        if(ch[0].action != NORM)
                                ch[0].moving = 1;
                        ch[0].n = 0;
                }
                if(ch[0].moving)
                        not_move = 0;

                for(n = 1; n < stat[lev].ghosts + 1; ++n) {
                        if(!ch[n].moving && !ch[n].falling) {
                                if(ch[n].action != NORM) {
                                        if(ch[n].dir == UP && ch[n].up)
                                                ch[n].action = UP;
                                        else if(ch[n].dir == DOWN && ch[n].down)
                                                ch[n].action = DOWN;
                                        else if(ch[n].dir == LEFT && ch[n].left)
                                                ch[n].action = LEFT;
                                        else if(ch[n].dir == RIGHT && ch[n].right)
                                                ch[n].action = RIGHT;
                                        else
                                                ch[n].action = NORM;
                                }

                                while(1) {
                                        n2 = rand() % 4;
                                        if(ch[n].action != NORM) {
                                                if(n2 == 0 && ch[n].dir == DOWN)
                                                        continue;
                                                if(n2 == 1 && ch[n].dir == UP)
                                                        continue;
                                                if(n2 == 2 && ch[n].dir == RIGHT)
                                                        continue;
                                                if(n2 == 3 && ch[n].dir == LEFT)
                                                        continue;
                                        }
                                        break;
                                }
                                if(n2 == 0 && ch[n].up)
                                        ch[n].action = UP;
                                if(n2 == 1 && ch[n].down)
                                        ch[n].action = DOWN;
                                if(n2 == 2 && ch[n].left)
                                        ch[n].action = LEFT;
                                if(n2 == 3 && ch[n].right)
                                        ch[n].action = RIGHT;
                                if(ch[n].action != NORM)
                                        ch[n].moving = 1;
                                ch[n].n = 0;
                        }
                }

                for(n = 0; n < stat[lev].ghosts + 1; ++n) {
                        if(n == 0 && ch[0].dead)
                                continue;
                        if(ch[n].moving) {
                                if(ch[n].action == UP) {
                                        ch[n].dir = UP;
                                        ch[n].y -= ch[n].movspeed;
                                }
                                if(ch[n].action == DOWN) {
                                        ch[n].dir = DOWN;
                                        ch[n].y += ch[n].movspeed;
                                }
                                if(ch[n].action == RIGHT) {
                                        ch[n].dir = RIGHT;
                                        ch[n].x += ch[n].movspeed;
                                }
                                if(ch[n].action == LEFT) {
                                        ch[n].dir = LEFT;
                                        ch[n].x -= ch[n].movspeed;
                                }
                                ch[n].n += ch[n].movspeed;
                                if(ch[n].n >= 32)
                                        ch[n].moving = 0;
                        }
                        else if(ch[n].falling) {
                                if(ch[n].fallcount > 0) {
                                        ch[n].y += 8;
                                        if(ch[n].falling == 1)
                                                ch[n].x += 6;
                                        else
                                                ch[n].x -= 6;
                                        --ch[n].fallcount;
                                }
                                else
                                        ch[n].falling = 0;
                        }
                }

                if(!keypress[6]) {
                        if(ch[0].x > sx + 181 && sx < 32*stat[lev].x - 320) {
                                if(ch[0].x - (sx + 181) >= scrollspeed) {
                                        sx += scrollspeed;
                                }
                                else {
                                        sx += ch[0].x - (sx + 181);
                                }
                        }
                        else if(ch[0].x < sx + 106 && sx > 0) {
                                if((sx + 106) - ch[0].x >= scrollspeed) {
                                        sx -= scrollspeed;
                                }
                                else {
                                        sx -= (sx + 106) - ch[0].x;
                                }
                        }
                        if(ch[0].y > sy + 123-32 && sy < 32*stat[lev].y - 192) {
                                if(ch[0].y - (sy + 123-32) >= scrollspeed) {
                                        sy += scrollspeed;
                                }
                                else {
                                        sy += ch[0].y - (sy + 123-32);
                                }
                        }
                        else if(ch[0].y < sy + 65 && sy > 0) {
                                if((sy + 65) - ch[0].y >= scrollspeed) {
                                        sy -= scrollspeed;
                                }
                                else {
                                        sy -= (sy + 65) - ch[0].y;
                                }
                        }
                }
                if(sx > 32*stat[lev].x - 320)
                        sx = 32*stat[lev].x - 320;
                if(sx < 0)
                        sx = 0;
                if(sy > 32*stat[lev].y - 192)
                        sy = 32*stat[lev].y - 192;
                if(sy < 0)
                        sy = 0;

                if(stablizer) {
                        ++game_loops;
                        now_time = get_ticks();
                        ticks = (now_time - start_time) / 14;

                        if(ticks > game_loops) {
                                skip_draw = 1;
                        }
                        else if(ticks < game_loops) {
                                while(ticks < game_loops) {
                                        now_time = get_ticks();
                                        ticks = (now_time - start_time) / 14;
                                }
                        }
                        if(game_loops == ticks && game_loops > 500) {
                                game_loops = 0;
                                start_time = get_ticks();
                        }
                }
        }

end:
        // Fade to white

        for(n = 0; n < 64; ++n) {
                for(n2 = 0; n2 < 256; ++n2) {
                        if(gamepal[n2 * 3] < 63)
                                ++gamepal[n2 * 3];
                        if(gamepal[n2 * 3 + 1] < 63)
                                ++gamepal[n2 * 3 + 1];
                        if(gamepal[n2 * 3 + 2] < 63)
                                ++gamepal[n2 * 3 + 2];
                }
                wait_r();
                setpal(gamepal);
                zap32(dbuf);
        }

        // Fade to black

        /*for(n = 0; n < 64; ++n) {
                for(n2 = 0; n2 < 256; ++n2) {
                        if(gamepal[n2 * 3] > 0)
                                --gamepal[n2 * 3];
                        if(gamepal[n2 * 3 + 1] > 0)
                                --gamepal[n2 * 3 + 1];
                        if(gamepal[n2 * 3 + 2] > 0)
                                --gamepal[n2 * 3 + 2];
                }
                wait_r();
                setpal(gamepal);
                zap32(dbuf);
        }*/

        sdl_deinit();
}
